"""Test the CityJSON class"""

import pytest
import copy
from cjio import cityjson
from math import isclose
import json


class TestCityJSON:
    def test_subset_ids(self, zurich_subset):
        # Parent ID
        subset = zurich_subset.get_subset_ids(
            ["UUID_583c776f-5b0c-4d42-9c37-5b94e0c21a30"]
        )
        expected = [
            "UUID_583c776f-5b0c-4d42-9c37-5b94e0c21a30",
            "UUID_60ae78b4-7632-49ca-89ed-3d1616d5eb80",
            "UUID_5bd1cee6-b3f0-40fb-a6ae-833e88305e31",
        ]
        assert set(expected).issubset(subset.j["CityObjects"])
        # Child ID
        subset2 = zurich_subset.get_subset_ids(
            ["UUID_60ae78b4-7632-49ca-89ed-3d1616d5eb80"]
        )
        expected = []
        assert set(expected).issubset(set(subset2.j["CityObjects"]))

    def test_subset_bbox(self, delft):
        cm = delft
        extent = cm.j["metadata"]["geographicalExtent"]
        bbox = [
            extent[0],
            extent[1],
            extent[3] - ((extent[3] - extent[0]) / 2),
            extent[4] - ((extent[4] - extent[1]) / 2),
        ]
        subset = cm.get_subset_bbox(bbox)
        for coid in subset.j["CityObjects"]:
            centroid = subset.get_centroid(coid)
            if centroid is not None:
                assert (
                    (centroid[0] >= bbox[0])
                    and (centroid[1] >= bbox[1])
                    and (centroid[0] < bbox[2])
                    and (centroid[1] < bbox[3])
                )

    def test_subset_bbox_loop(self, delft):
        """Issue #10"""
        _ = delft.update_bbox()
        subs_box = (
            84873.68845606346,
            447503.6748565406,
            84919.65679078053,
            447548.4091420035,
        )
        nr_cos = []
        for i in range(4):
            s = delft.get_subset_bbox(subs_box)
            nr_cos.append(len(s.j["CityObjects"]))
        _f = nr_cos[0]
        assert all(i == _f for i in nr_cos)

    def test_subset_random(self, zurich_subset):
        subset = zurich_subset.get_subset_random(10)
        cnt = sum(
            1 for co in subset.j["CityObjects"].values() if co["type"] == "Building"
        )
        assert cnt == 10

    def test_subset_cotype(self, delft):
        subset = delft.get_subset_cotype(("Building", "LandUse"))
        types = [
            "LandUse",
            "Building",
            "BuildingPart",
            "BuildingInstallation",
            "BuildingConstructiveElement",
            "BuildingFurniture",
            "BuildingStorey",
            "BuildingRoom",
            "BuildingUnit",
        ]

        for co in subset.j["CityObjects"]:
            assert subset.j["CityObjects"][co]["type"] in types

    def test_calculate_bbox(self):
        """Test the calculate_bbox function"""

        data = {"vertices": [[0, 0, 0], [1, 1, 1]]}

        cm = cityjson.CityJSON(j=data)
        bbox = cm.calculate_bbox()

        assert bbox == [0, 0, 0, 1, 1, 1]

    def test_calculate_bbox_with_transform(self):
        """Test the calculate_bbox function"""

        data = {
            "vertices": [[0, 0, 0], [1, 1, 1]],
            "transform": {"scale": [0.001, 0.001, 0.001], "translate": [100, 100, 100]},
        }

        cm = cityjson.CityJSON(j=data)
        bbox = cm.calculate_bbox()

        assert bbox == [100, 100, 100, 100.001, 100.001, 100.001]

    def test_de_compression(self, delft):
        cm = copy.deepcopy(delft)
        assert cm.decompress()
        cm2 = copy.deepcopy(cm)
        cm.compress(3)
        assert cm.j["transform"]["scale"][0] == 0.001
        assert len(delft.j["vertices"]) == len(cm.j["vertices"])
        v1 = cm2.j["vertices"][0][0]
        v2 = cm.j["vertices"][0][0]
        assert isinstance(v1, float)
        assert isinstance(v2, int)

    def test_de_compression_2(self, cube):
        cubec = copy.deepcopy(cube)
        cubec.decompress()
        assert cube.j["vertices"][0][0] == cubec.j["vertices"][0][0]
        assert cubec.compress(2)
        assert len(cube.j["vertices"]) == len(cubec.j["vertices"])

    def test_reproject(self, delft_1b):
        cm = copy.deepcopy(delft_1b)
        cm.reproject(4937)  # -- z values should stay the same
        x = (
            cm.j["vertices"][0][0] * cm.j["transform"]["scale"][0]
            + cm.j["transform"]["translate"][0]
        )
        y = (
            cm.j["vertices"][0][1] * cm.j["transform"]["scale"][1]
            + cm.j["transform"]["translate"][1]
        )
        z = (
            cm.j["vertices"][0][2] * cm.j["transform"]["scale"][2]
            + cm.j["transform"]["translate"][2]
        )
        print(x, y, z)
        assert x == pytest.approx(52.011288184126094)
        assert y == pytest.approx(4.36772776578513)
        assert z == pytest.approx(49.50418078666017)
        assert isclose(
            cm.j["metadata"]["geographicalExtent"][5]
            - cm.j["metadata"]["geographicalExtent"][2],
            6.1,
            abs_tol=0.001,
        )

        cm.reproject(7415)
        x = (
            cm.j["vertices"][0][0] * cm.j["transform"]["scale"][0]
            + cm.j["transform"]["translate"][0]
        )
        y = (
            cm.j["vertices"][0][1] * cm.j["transform"]["scale"][1]
            + cm.j["transform"]["translate"][1]
        )
        z = (
            cm.j["vertices"][0][2] * cm.j["transform"]["scale"][2]
            + cm.j["transform"]["translate"][2]
        )
        print(x, y, z)
        x_d = (
            delft_1b.j["vertices"][0][0] * delft_1b.j["transform"]["scale"][0]
            + delft_1b.j["transform"]["translate"][0]
        )
        y_d = (
            delft_1b.j["vertices"][0][1] * delft_1b.j["transform"]["scale"][1]
            + delft_1b.j["transform"]["translate"][1]
        )
        z_d = (
            delft_1b.j["vertices"][0][2] * delft_1b.j["transform"]["scale"][2]
            + delft_1b.j["transform"]["translate"][2]
        )
        assert x == pytest.approx(x_d)
        assert y == pytest.approx(y_d)
        assert z == pytest.approx(z_d)

    def test_convert_to_stl(self, delft):
        cm = copy.deepcopy(delft)
        _ = cm.export2stl(sloppy=True)

    def test_triangulate(self, materials):
        cm = materials
        cm.triangulate(sloppy=False)

    def test_is_triangulate(self, triangulated):
        cm = triangulated
        assert cm.is_triangulated()

    def test_convert_to_jsonl(self, rotterdam_subset):
        cm = copy.deepcopy(rotterdam_subset)
        jsonl = cm.export2jsonl()
        assert jsonl is not None
        jsonl.seek(0)
        for line in jsonl.readlines():
            data = json.loads(line)

            assert "CityObjects" in data

    def test_filter_lod(self, multi_lod):
        cm = multi_lod
        cm.filter_lod("1.3")
        for coid in cm.j["CityObjects"]:
            if "geometry" in cm.j["CityObjects"][coid]:
                for geom in cm.j["CityObjects"][coid]["geometry"]:
                    assert geom["lod"] == "1.3"

    def test_merge_materials(self, materials_two):
        """Testing #100
        Merging two files with materials. One has the member 'values', the other has the
        member 'value' on their CityObjects.
        """
        cm1, cm2 = materials_two
        # cm1 contains the CityObject with 'value'. During the merge, the Material Object
        # from cm1 is appended to the list of Materials in cm2
        assert cm2.merge([cm1])
        assert len(cm2.j["CityObjects"]) == 4
        # The value of 'value' in the CityObject from cm1 must be updated to point to the
        # correct Material Object in the materials list
        assert (
            cm2.j["CityObjects"]["NL.IMBAG.Pand.0518100001755018-0"]["geometry"][0][
                "material"
            ]["default"]["value"]
            == 1
        )



import os.path
from copy import deepcopy

import pytest

from cjio import errors


def test_get_textures_location_subdir(rotterdam_subset, data_dir):
    """Textures are in a subdirectory to the city model"""
    d = rotterdam_subset.get_textures_location()
    loc = os.path.join(data_dir, "rotterdam", "appearances")
    assert d == loc


def test_get_textures_location_same(dummy, data_dir):
    """Textures are in same location with the city model"""
    d = dummy.get_textures_location()
    loc = os.path.join(data_dir, "dummy")
    assert d == loc


def test_update_textures_relative(rotterdam_subset, data_dir):
    r = deepcopy(rotterdam_subset)
    npath = os.path.join(data_dir, "rotterdam", "textures")
    r.update_textures_location(npath, relative=True)
    dirs = []
    for t in r.j["appearance"]["textures"]:
        dirs.append(os.path.dirname(t["image"]))
    assert all(p == "textures" for p in dirs)


def test_update_textures_absolute(rotterdam_subset, data_dir):
    r = deepcopy(rotterdam_subset)
    npath = os.path.join(data_dir, "rotterdam", "textures")
    r.update_textures_location(npath, relative=False)
    dirs = []
    for t in r.j["appearance"]["textures"]:
        dirs.append(os.path.dirname(t["image"]))
    assert all(p == npath for p in dirs)


def test_update_textures_nodir(rotterdam_subset, data_dir):
    r = deepcopy(rotterdam_subset)
    npath = os.path.join(data_dir, "rotterdam", "not_existing_dir")
    with pytest.raises(NotADirectoryError):
        r.update_textures_location(npath, relative=False)


def test_update_textures_url(rotterdam_subset):
    r = deepcopy(rotterdam_subset)
    npath = "http://www.balazs.com/images"
    r.update_textures_location(npath, relative=False)
    dirs = []
    for t in r.j["appearance"]["textures"]:
        dirs.append(os.path.dirname(t["image"]))
    assert all(p == npath for p in dirs)


def test_update_textures_none(dummy_noappearance):
    with pytest.raises(errors.CJInvalidOperation):
        dummy_noappearance.update_textures_location("somepath", relative=True)


def test_remove_textures(rotterdam_subset):
    cm = rotterdam_subset
    cm.remove_textures()
    assert "appearance" not in cm.j
    assert all(
        "texture" not in geom
        for co in cm.j["CityObjects"].values()
        for geom in co["geometry"]
    )



import os
import os.path
from click.testing import CliRunner
from cjio import cjio, cityjson, __version__
import pytest


class TestCLI:
    def test_version_cli(self):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=["--version"])

        assert result.exit_code == 0
        assert __version__ in result.output

    def test_help_cli(self):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=["--help"])
        assert result.exit_code == 0
        assert "Options" in result.output

    def test_help_subcommand_cli(self):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=["validate", "--help"])
        assert result.exit_code == 0
        assert "Options" in result.output

    def test_attribute_remove_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "attribute_remove.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[sample_input_path, "attribute_remove", "bgt_status", "save", p_out],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_attribute_rename_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "attribute_rename.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                sample_input_path,
                "attribute_rename",
                "hoek",
                "angle",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_crs_assign_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "crs_assign.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "crs_assign", "4326", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_crs_reproject_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "crs_reproject.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                sample_input_path,
                "crs_reproject",
                "--digit",
                "7",
                "4979",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_crs_translate_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "crs_translate.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                sample_input_path,
                "crs_translate",
                "--minxyz",
                "-1",
                "-1",
                "-1",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_obj_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "delft.obj")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "export", "obj", p_out]
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_glb_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "delft.glb")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "export", "glb", p_out]
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_b3dm_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "delft.b3dm")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "export", "b3dm", p_out]
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_stl_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "delft.stl")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "export", "stl", p_out]
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_jsonl_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "delft.city.jsonl")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "export", "jsonl", p_out]
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_export_wrong_file_cli(self, wrong_input_path):
        p_out = os.path.join("tests", "data", "delft_non.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[wrong_input_path, "export", "jsonl", p_out]
        )

        assert not os.path.exists(p_out)
        assert result.exit_code != 0

    def test_info_cli(self, sample_input_path):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=[sample_input_path, "info"])

        assert result.exit_code == 0

    def test_lod_filter_cli(self, multi_lod_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "filtered_lod.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[multi_lod_path, "lod_filter", "1.2", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)
        assert "1.2" in result.output

        os.remove(p_out)

    def test_materials_remove_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "materials_remove.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[rotterdam_subset_path, "materials_remove", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_merge_cli(self, sample_input_path, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "merge.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[sample_input_path, "merge", rotterdam_subset_path, "save", p_out],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_metadata_extended_remove_cli(
        self, sample_with_ext_metadata_input_path, data_output_dir
    ):
        p_out = os.path.join(data_output_dir, "cleaned_file.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                sample_with_ext_metadata_input_path,
                "metadata_extended_remove",
                "save",
                p_out,
            ],
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_metadata_get_cli(self, sample_input_path):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=[sample_input_path, "metadata_get"])

        assert result.exit_code == 0

    def test_print_cli(self, sample_input_path):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=[sample_input_path, "print"])
        assert result.exit_code == 0
        assert result.exit_code == 0
        assert "CityJSON" in result.output
        assert "EPSG" in result.output

    def test_save_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "save.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "save", "--indent", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_save_with_texture_cli(
        self, rotterdam_subset_path, data_output_dir, temp_texture_dir
    ):
        p_out = os.path.join(data_output_dir, "save_texture.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "save",
                "--textures",
                temp_texture_dir,
                "--indent",
                p_out,
            ],
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        result2 = runner.invoke(
            cjio.cli,
            args=[
                p_out,
                "textures_locate",
            ],
        )
        assert result2.exit_code == 0
        assert temp_texture_dir in result2.output

        os.remove(p_out)

    def test_subset_id_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "subset.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "subset",
                "--id",
                "{23D8CA22-0C82-4453-A11E-B3F2B3116DB4}",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_subset_bbox_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "subset_bbox.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "subset",
                "--bbox",
                90970,
                435620,
                91000,
                435650,
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_subset_radius_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "subset_radius.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "subset",
                "--radius",
                90970,
                435620,
                20.0,
                "--exclude",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_subset_random_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "subset_random.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "subset",
                "--random",
                3,
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_subset_cotype_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "subset_cotype.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                sample_input_path,
                "subset",
                "--cotype",
                "Bridge",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_textures_locate_cli_no_textures(self, sample_input_path):
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=[sample_input_path, "textures_locate"])

        assert result.exit_code == 0
        assert "This file does not have textures" in result.output

    def test_textures_locate_cli_with_textures(self, rotterdam_subset_path):
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[rotterdam_subset_path, "textures_locate"]
        )

        assert result.exit_code == 0
        assert "rotterdam/appearances" in result.output

    def test_textures_remove_cli(self, rotterdam_subset_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "textures_remove.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[rotterdam_subset_path, "textures_remove", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        result2 = runner.invoke(cjio.cli, args=[p_out, "textures_locate"])

        assert result2.exit_code == 0
        assert "This file does not have textures" in result2.output

        os.remove(p_out)

    def test_textures_update_cli(
        self, rotterdam_subset_path, data_output_dir, temp_texture_dir
    ):
        p_out = os.path.join(data_output_dir, "updated_textures.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "textures_update",
                temp_texture_dir,
                "save",
                p_out,
            ],
        )
        assert result.exit_code == 0
        assert os.path.exists(p_out)

        result2 = runner.invoke(cjio.cli, args=[p_out, "textures_locate"])

        assert result2.exit_code == 0
        assert temp_texture_dir in result2.output

        os.remove(p_out)

    def test_triangulate_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "triangulated.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "triangulate", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_triangulate_sloppy_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "triangulated.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "triangulate", "--sloppy", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_upgrade_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "upgrade.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "upgrade", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_validate_cli(self, sample_input_path):
        if not cityjson.MODULE_CJVAL_AVAILABLE:  # pragma: no cover
            pytest.skip("cjvalpy module not available")
        runner = CliRunner()
        result = runner.invoke(cjio.cli, args=[sample_input_path, "validate"])

        assert result.exit_code == 0

    def test_vertices_clean_cli(self, sample_input_path, data_output_dir):
        p_out = os.path.join(data_output_dir, "clean.city.json")
        runner = CliRunner()
        result = runner.invoke(
            cjio.cli, args=[sample_input_path, "vertices_clean", "save", p_out]
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_chained_commands_cli(self, rotterdam_subset_path, data_output_dir):
        """
        Test chaining multiple commands to ensure process_pipeline is invoked correctly.
        """
        p_out = os.path.join(data_output_dir, "pipeline_output.city.json")
        runner = CliRunner()

        result = runner.invoke(
            cjio.cli,
            args=[
                rotterdam_subset_path,
                "subset",
                "--id",
                "{23D8CA22-0C82-4453-A11E-B3F2B3116DB4}",
                "vertices_clean",
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_off_input_cli(self, sample_off_file, data_output_dir):
        """
        Test that the CLI works with a .off input file."""
        p_out = os.path.join(data_output_dir, "box.city.json")
        runner = CliRunner()

        result = runner.invoke(
            cjio.cli,
            args=[
                sample_off_file,
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_poly_input_cli_0_index(self, sample_poly_0_index_file, data_output_dir):
        """
        Test that the CLI works with a .poly input file."""
        p_out = os.path.join(data_output_dir, "cube.city.json")
        runner = CliRunner()

        result = runner.invoke(
            cjio.cli,
            args=[
                sample_poly_0_index_file,
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_poly_input_cli_1_index(self, sample_poly_1_index_file, data_output_dir):
        """
        Test that the CLI works with a .poly input file."""
        p_out = os.path.join(data_output_dir, "cube.city.json")
        runner = CliRunner()

        result = runner.invoke(
            cjio.cli,
            args=[
                sample_poly_1_index_file,
                "save",
                p_out,
            ],
        )

        assert result.exit_code == 0
        assert os.path.exists(p_out)

        os.remove(p_out)

    def test_wrong_extension_cli(self, sample_wrong_suffix):
        """
        Test that the CLI gives a warning when the file extension is not supported.
        """
        runner = CliRunner()

        result = runner.invoke(
            cjio.cli,
            args=[
                sample_wrong_suffix,
                "subset",
                "--id",
                "{23D8CA22-0C82-4453-A11E-B3F2B3116DB4}",
                "vertices_clean",
                "save",
                "output.city.json",
            ],
        )
        assert result.exit_code != 0
        assert "File type not supported" in result.output



import warnings


class CJInvalidOperation(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return "InvalidOperation: %s" % self.msg


class CJInvalidVersion(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return "InvalidVersion: %s" % self.msg


class CJWarning(Warning):
    def __init__(self, msg):
        super().__init__()
        self.msg = msg

    def __str__(self):
        return self.msg

    def warn(self):
        warnings.warn(self)



import pytest
import os

from cjio import convert, cityjson


@pytest.fixture(
    scope="function", params=[("rotterdam", "rotterdam_subset.json"), ("delft.json",)]
)
def several_cms(data_dir, request):
    p = os.path.join(data_dir, *request.param)
    with open(p, "r") as f:
        yield cityjson.CityJSON(file=f)


class TestGltf:
    @pytest.mark.slow
    def test_convert_to_glb(self, several_cms):
        _ = several_cms.export2glb()

    def test_debug_den_haag_glb(self, data_dir):
        p = os.path.join(data_dir, "DH_01_subs.city.json")
        with open(p, "r") as f:
            cm = cityjson.CityJSON(file=f)
        _ = cm.export2glb()

    def test_debug_delft_glb(self, data_dir, data_output_dir):
        # CityJSON v1.1
        p = os.path.join(data_dir, "DH_01_subs.city.json")
        with open(p, "r") as f:
            cm = cityjson.CityJSON(file=f)
        glb = cm.export2glb(do_triangulate=False)
        glb.seek(0)
        with open(f"{data_output_dir}/DH_01_subs.glb", mode="wb") as bo:
            bo.write(glb.getvalue())


class TestB3dm:
    @pytest.mark.slow
    def test_convert_to_b3dm(self, delft):
        glb = delft.export2glb()
        _ = convert.to_b3dm(delft, glb)



from setuptools import setup
from pathlib import Path

CURRENT_DIR = Path(__file__).parent

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name="cjio",
    version="0.10.0",
    description="CLI to process and manipulate CityJSON files",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/cityjson/cjio",
    author="Hugo Ledoux, Balázs Dukai",
    author_email="h.ledoux@tudelft.nl, balazs.dukai@3dgi.nl",
    python_requires=">=3.6",
    packages=["cjio"],
    # include_package_data=True,
    license="MIT",
    classifiers=[
        # https://pypi.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: GIS",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    install_requires=["numpy", "Click>=8.1.0"],
    extras_require={
        "develop": [
            "pytest",
            "bump2version",
            "coverage",
        ],
        "export": ["pandas", "mapbox-earcut", "triangle2"],
        "validate": ["cjvalpy>=0.3.0"],
        "reproject": ["pyproj>=3.0.0"],
    },
    entry_points="""
        [console_scripts]
        cjio=cjio.cjio:cli
    """,
)
